import time

class PID_controller():

    def __init__(self, kp, ki, kd):
        self.Kp = kp
        self.Ki = ki
        self.Kd = kp
        
        self.Error =0
        self.Prev_Error =0
        
        self.Prev_Time =0
        self.Time =0
        
        self.Derivative =0
        self.Integral =0
    
     
    def Compute(self, setpoint, feedback, debug=False):
        self.Time = time.time()
        
        #calculate dt
        dt = (self.Time - self.Prev_Time)
         
        #determine Error
        self.Error = setpoint-feedback
        
        #Proportional
        P = self.Kp * self.Error
        
        #Derivative  #Secant Line
         # original: fd(x) = (f(x+T)-f(x))/T
         # using:    fd(x) = (f(x) - f(x-T))/T
        self.Derivative = (self.Error - self.Prev_Error)/dt
        D = self.Kd * self.Derivative  
        
        #Integral #Trapazoidal Rule
         #f(k) = T*(f(k) + f(k+1))/2)
        self.Integral = dt*((self.Error + self.Prev_Error)/2)
        I = self.Ki * self.Integral
        
        
        if debug == True:
            print('Error: {0:4f} PID: {1:4f} P: {2:4f} I: {3:4f}  D: {4:4f}'.format(self.Error,P+I+D,P,I,D))
        self.Prev_Time  =  self.Time
        self.Prev_Error =  self.Error
        return P+I+D
