import processing.net.*;
Client PythonProg;
String DataStr;
PFont OurFont; 
OurPlot RotationPlot;
OurPlot PositionPlot;
OurPlot JoystickPlot;
OurPlot PIDPlot;
OurPlot PosPIDPlot;
PShape Mode0Square;
PShape Mode1Square;
PShape Mode2Square;

PShape PendCalibSquare;
PShape LeftCalibSquare;
PShape RightCalibSquare;

PShape CraneRail;
PShape CraneCar;
PShape CranePendulum;

PShape PosSelector;



int JoyData = 0;

int PendCalibLock = 0;
String PendCalib;

int LeftCalibLock = 0;
String LeftCalib;

int RightCalibLock = 0;
String RightCalib;

float CranePixelDistanceScaler;

void setup()
{
  size(1280,1024);  //our final size needs to be 1280, 1024
  background(255);
  fill(200);
  frameRate(120);
  // Must output data to socket 50000 for processing to pickup.
    PythonProg = new Client(this,"127.0.0.1",50000);
  RotationPlot = new OurPlot(800,220,400,200,0,200,-90,90,"Theta", color(255,0,0));
  PositionPlot = new OurPlot(800,450,400,200,0,200,-60,60 ,"X Position", color(0,0,255));
  JoystickPlot = new OurPlot(800,700,400,200,0,200,-1,300,"JoyStick Input", color(0,255,0));
       PIDPlot = new OurPlot(800,930,400,200,0,200,-1,300,"Angle Compensating PID", color(255,0,255));
       PosPIDPlot = new OurPlot(350,800,400,200,0,200,-1,300,"Position Compensating PID", color(0,255,255));
                // Xpos, Ypos, XSize, YSize, TMin, TMax, Ymin, Ymax, Label
     
  PendCalibSquare  = createShape(RECT,10,740,160,20, 0, 0, 0, 0);
  LeftCalibSquare  = createShape(RECT,10,770,160,20, 0, 0, 0, 0);
  RightCalibSquare = createShape(RECT,10,800,160,20, 0, 0, 0, 0);
  Mode0Square = createShape(RECT,10,850,200,100, 0, 0, 0, 0);
  Mode1Square = createShape(RECT,220,850,200,100, 0, 0, 0, 0);
  Mode2Square = createShape(RECT,430,850,200,100, 0, 0, 0, 0);
  
  CraneRail = createShape(RECT,50,140,600,10, 0, 0, 0, 0);
  CraneRail.setFill(color(100,100,100));
  
  print("PythonComms Initialized!");
  OurFont = createFont("Arial",16,false);  
}

// this is actually main() it has to be called draw though, 
void draw()
{
  if(PythonProg.available() > 0)
  {
     background(255);
     DataStr = PythonProg.readString();
    
     textFont(OurFont,16);
     fill(0);
     
     
     text(DataStr,10,650);
    
     String[] DataStrList_Newlines = split(DataStr,'\n');
     String[] DataStrList_Commas  = trim(split(DataStrList_Newlines[0],','));
     
     text(-int(DataStrList_Commas[11]),20,20);
     
     float RotationData = -(float(DataStrList_Commas[1])*0.087);  // 360/(1024*4) = 0.08789
     float PositionData = -(float(DataStrList_Commas[0])/766);    // 766 pulses per cm?? (Not really need better approx.)
     JoystickPlot.DrawPlot_NewData(-int(DataStrList_Commas[2]) + 127);
     RotationPlot.DrawPlot_NewData(RotationData);
     PositionPlot.DrawPlot_NewData(PositionData);
     PIDPlot.DrawPlot_NewData(-int(DataStrList_Commas[3]) +127);
     
     shape(CraneRail);
    
     CranePixelDistanceScaler = 600/( float(DataStrList_Commas[9]) - float(DataStrList_Commas[8])); 
     CraneCar = createShape(RECT,330+CranePixelDistanceScaler*float(DataStrList_Commas[0]),120,40,40, 0, 0, 0, 0);
     CraneCar.setFill(color(50,50,50));
     CraneCar.setStroke(color(50,50,50)); 
     shape(CraneCar);
     
     CranePendulum = createShape(RECT,-2.5,0,5,400, 0, 0, 0, 0);
     CranePendulum.setFill(color(200,200,200));
     CranePendulum.setStroke(color(200,200,200));
     CranePendulum.translate(350+CranePixelDistanceScaler*float(DataStrList_Commas[0]),160);
     CranePendulum.rotate(float(DataStrList_Commas[1])*0.00153); // 2pi/(1024*4) = 0.00153
                                                   
     shape(CranePendulum);
     
    // DRAW MODE0 SQUARE
    if (int(DataStrList_Commas[4]) == 0) 
    { 
      Mode0Square.setFill(color(50,255,50));
    }
    else
    {
      Mode0Square.setFill(color(127,127,127));
    }
    shape(Mode0Square);
    textSize(24);
    text("Manual Mode",30,900);
    
    // DRAW MODE1 SQUARE
    if (int(DataStrList_Commas[4]) == 1) 
    { 
      Mode1Square.setFill(color(50,255,50));
    }
    else
    {
      Mode1Square.setFill(color(127,127,127));
    }
    shape(Mode1Square);
    textSize(24);
    text("Semi_Auto",250,900);
    
    // DRAW MODE2 SQUARE
    if (int(DataStrList_Commas[4]) == 2) 
    { 
      Mode2Square.setFill(color(50,255,50));
      
      PosSelector = createShape(TRIANGLE,-30,20,0,40,30,20);
      PosSelector.setFill(color(20,200,20)); 
      PosSelector.setStroke(color(20,200,20));
      PosSelector.translate(50+(float(DataStrList_Commas[10])-float(DataStrList_Commas[8]))*CranePixelDistanceScaler,50);
      shape(PosSelector);
      PosPIDPlot.DrawPlot_NewData(-int(DataStrList_Commas[11])+127);
    }
    else
    {
      Mode2Square.setFill(color(127,127,127));
    }
    shape(Mode2Square);
    textSize(24);
    text("Automatic",450,900);
   
    if(PendCalibLock == 0) 
    {
      PendCalib = DataStrList_Commas[5];
      if(int(DataStrList_Commas[5]) == 0 )
      {
        PendCalibSquare.setFill(color(255,0,0));  
      }
      if(int(DataStrList_Commas[5]) > 0 && int(DataStrList_Commas[5]) < 200)
      {
        PendCalibSquare.setFill(color(255,255,50));  
      }
      if(int(DataStrList_Commas[5]) > 200)
      {
        PendCalibSquare.setFill(color(50,255,50));  
        PendCalibLock =1;
        PendCalib = "Pendulum Rest Calibrated";
      }
    }
    shape(PendCalibSquare); 
    textSize(11);
    text(PendCalib,20,755);
    
    
    if(LeftCalibLock == 0) 
    {
      LeftCalib = DataStrList_Commas[6];
      if(int(DataStrList_Commas[6]) == 0 )
      {
        LeftCalibSquare.setFill(color(255,0,0));  
      }
      if(int(DataStrList_Commas[6]) > 0 && int(DataStrList_Commas[5]) < 200)
      {
        LeftCalibSquare.setFill(color(255,255,50));  
      }
      if(int(DataStrList_Commas[6]) > 200)
      {
        LeftCalibSquare.setFill(color(50,255,50));  
        LeftCalibLock =1;
        LeftCalib = "Left Bound Calibrated";
      }
    }
    shape(LeftCalibSquare);  
    textSize(11);
    text(LeftCalib,20,785);
    
    
    if(RightCalibLock == 0) 
    {
      RightCalib = DataStrList_Commas[7];
      if(int(DataStrList_Commas[7]) == 0 )
      {
        RightCalibSquare.setFill(color(255,0,0));  
      }
      if(int(DataStrList_Commas[7]) > 0 && int(DataStrList_Commas[5]) < 200)
      {
        RightCalibSquare.setFill(color(255,255,50));  
      }
      if(int(DataStrList_Commas[7]) > 200)
      {
        RightCalibSquare.setFill(color(50,255,50));  
        RightCalibLock =1;
        RightCalib = "Right Bound Calibrated";
      }
    }
    shape(RightCalibSquare);
    textSize(11);
    text(RightCalib,20,815);
    text("All calibrations must be done before using Automatic Mode",20,835);
  }
}

class OurPlot
{
  // Initialization Data 
  private
  int m_Xpos;
  int m_Ypos;
  int m_XSize;
  int m_YSize;
  int m_TMin;
  int m_TMax;
  int m_YMin;
  int m_YMax;
  
  float m_YMiddle; 
  
  int PlotTime =0; // Iterator for position in plot
  float[] PlotData; // Actual Matrix of Data in plot
 
  float T_Data_to_pixel_scale;
  float Y_Data_to_pixel_scale;
  
  String m_Label;
  color m_LineColor; 
  
  PFont PlotFont;
  
  public
  
  // initialize plot. 
  OurPlot(int Xpos, int Ypos, int XSize, int YSize, int TMin, int TMax, int YMin, int YMax, String Label, color LineColor)
  {
     m_Xpos = Xpos;
     m_Ypos = Ypos;
     m_XSize = XSize;
     m_YSize = YSize;
     m_TMin = TMin;
     m_TMax = TMax;
     m_YMin = YMin;
     m_YMax = YMax;
     m_Label = Label;
     m_LineColor = LineColor;
     m_YMiddle = (m_Ypos-(float)(m_YSize)/2);
   
     T_Data_to_pixel_scale = ((float)XSize/(TMax-TMin)); // This gives us X pixels per Time Point
     Y_Data_to_pixel_scale = ((float)YSize/(YMax-YMin)); // this gives us Y pixels per Data Point
     
     PlotData = new float[TMax+1]; 
     
     PlotFont = createFont("Arial",12,false);
  }
  
  void DrawPlot_NewData(float NewYData)
  {
    stroke(50);
    // vertical line
    line(m_Xpos, m_Ypos, m_Xpos, m_Ypos-m_YSize);    
    //horizontal line
    line(m_Xpos, m_YMiddle, m_Xpos+m_XSize, m_YMiddle);
    // Plot Lable
    text(m_Label,m_Xpos,m_Ypos-m_YSize);
    
    if(PlotTime == m_TMax) // If we have reached the end of the plot, 
    {
      PlotData[0] = PlotData[m_TMax];
      for(int t = 1; t <= PlotTime; t++) // forget the previous data
      {
        PlotData[t] = 0;
      }
      PlotTime = 0;
    }

     PlotTime++;
     PlotData[PlotTime] = (NewYData * Y_Data_to_pixel_scale); 
     text(nf(NewYData,2,3),m_Xpos+30,m_Ypos);
     for(int t = 1; t <= PlotTime; t++) // Draw lines from 0 to PlotTime
     {
       // We Want to draw a line from PrevTisme to Now
       float TNowPos =  m_Xpos + (T_Data_to_pixel_scale*t);
       float TPrevPos = m_Xpos + (T_Data_to_pixel_scale*(t-1));
      
       float YNowPos =  m_YMiddle + (PlotData[t]) ;
       float YPrevPos = m_YMiddle + (PlotData[t-1]);
     
       stroke(m_LineColor);
       line(TNowPos, YNowPos, TPrevPos, YPrevPos);
     }
  }
}
