#!/usr/bin/python
import evdev
import spidev
import time
import socket
import os
from PID import PID_controller  

# Initialization for socket communication to PythonComms code
HOST = ''
PORT = 50000
Sock= socket.socket(socket.AF_INET, socket.SOCK_STREAM)
Sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
Sock.bind((HOST,PORT))
Sock.listen(1)

# Initialization for SPI communication to Microcontroller (Make sure it is plugged in correctly)
spi = spidev.SpiDev()
spi.open(0, 0)
#  (bus,device)
spi.max_speed_hz = 15200 # or 1.52kHz
#3.9MHz, 1953kHz, 976kHz, 488kHz, 244kHz, 122kHz, 61kHz, 30.5kHz, 15.2kHz, 7629kHz

# Initialization for Joystick input
joystick = evdev.InputDevice('/dev/input/event0')
print(joystick)


# Initialization for PID controllers, Please look at the PID.py and understand it before changing any of these values,
PendulumController = PID_controller(0.015,0.01,0.01)
PositionController = PID_controller(0.01,0.01,0.01)


# This function reads all recent joystick events, it needs to be called fairly often to be responsive
 # Right now it is only set to read the X axis of the Joystick, the trigger button and the two buttons on the top of the joystick.
def joystick_poll():
    out = -1
    up_btn = 0
    down_btn = 0
    trigger =0
    
    try:
        eventlist = joystick.read()
        for event in eventlist: ### search for a more efficent non-blocking way to read joystick events
            if event.code == 0: # JoystickXAxis
                if event.type == 3:
                    #print('XAxis',event.value)
                    Xdata = event.value
                    out = Xdata
##            if event.code == 1: # JoystickYAxis
##                if event.type == 3:
##                    #print('YAxis',event.value)
##                    Ydata = event.value
##                    break;
            if event.code == 288: #Trigger button
                if event.type == 1:
                    trigger = event.value
                    
            if event.code == 289: #Upward button
                if event.type == 1:
                    up_btn = event.value
             
            if event.code == 290: #Downward button
                if event.type == 1:
                    down_btn = event.value
    except:
        out = -1 # No Events this time..
    
    state_button = up_btn - down_btn;    
    if out > 127 and out < 137  :
        out = 127 # I am using 128 to 137 as a special value to signal 'GIVE ME __ DATA'
                  # So if the joystick were to send that value the microcontroller would be very confused.   
    return out,state_button,trigger

# Joystick variables
Out =0
PrevOut =127

StateSel_Prev = 0;
StateSel =0;
TriggerVal_Prev =0;
TriggerVal =0;

Mode = 0;

# Rotation variables
RawRotation =0
RawRotation_Prev =0
Rot_FirstNonZero = 0
Rotation =0
DegreesRotation = 0
RadiansRotation = 0

NewCenterCycles = 0
CenterCalibDone = 0

RotationByte0 =0
RotationByte1 =0

# Position variables
RawPosition =0
RawPosition_Prev = 0
Pos_FirstNonZero =0
Position =0

LeftLimit  =0
NewLeftLimitCycles = 0
LeftCalibDone =0

RightLimit =62722
NewRightLimitCycles = 0
RightCalibDone =0

PositionByte0 =0
PositionByte1 =0
PositionByte2 =0
PositionByte3 =0

PositionSelector=0;
SetPoint_PositionSelector=0;

MotionIN =0;


#Starts the PythonComms program for the Graphical interface
os.chdir("/home/pi/processing-3.3.7")      # --present used to fullscreen, --run should only run it in window
disp = os.popen("DISPLAY=:0 ./processing-java --sketch=/home/pi/Desktop/GantryCraneCode/PythonComms --present")
connection,client_address = Sock.accept() # CONNECT TO CRANE DISPLAY
print(client_address,' Connected!')    
 

while(1):


    # Get Joystick Data
    [Out,StateSel,TriggerVal] = joystick_poll()
    if Out != -1:
        PrevOut = Out

    # Mode Selection, Case statements do not exist in python :(
    if StateSel != StateSel_Prev: 
        if StateSel_Prev == 0:
            if StateSel == 1:
                Mode = Mode+1
                if Mode >= 2: # Do not let Mode out of Upper bounds
                    Mode = 1  # ^^^^^ TEMPORARILY DISABLE MODE 2! AUTOMATIC MODE NOT READY...
            if StateSel == -1:
                Mode = Mode-1
                if Mode <= -1: # Do not let Mode out of Lower bounds
                    Mode = 0
    StateSel_Prev = StateSel
    
    if Mode == 0:
        # do Mode 0 things
        # direct user control,
        MotionIN = PrevOut



    if Mode == 1:
        # do Mode 1 things

        # Right now, we let the PID controller act when the joystick is outside it's deadzone,
        #  But, we want to find a more robust solution to this, like scaling the PID controller to take more control
        #  when the pendulum angle is large, and give user more control when pendulum is calm.
        if PrevOut < 118:
            if PIDJoy < PrevOut:
                MergeValues = PIDJoy
            else:
                MergeValues = PrevOut
        
        elif PrevOut > 137:
            if PIDJoy > PrevOut:
                MergeValues = PIDJoy
            else:
                MergeValues = PrevOut
        else:
            MergeValues = PIDJoy
    
        MotionIN = MergeValues
    
    if Mode == 2:
        # do Mode 2 things

        # WE never got the Position PID tuned, and did not figure out
        # the proper way to use both PID loops at the same time,
        if CenterCalibDone == 1 and LeftCalibDone == 1 and RightCalibDone ==1:
            if   PrevOut < 118:
                PositionSelector = PositionSelector - -(PrevOut-118)*10  
            elif PrevOut > 137:
                PositionSelector = PositionSelector + (PrevOut-137)*10
            
            if   PositionSelector > RightLimit:
                PositionSelector = RightLimit
                
            elif PositionSelector < LeftLimit:
                PositionSelector = LeftLimit
            
            if TriggerVal == 1 and TriggerVal_Prev == 0:
                SetPoint_PositionSelector = PositionSelector
            TriggerVal_Prev = TriggerVal
            
            MotionIN = PosPIDJoy
        else:
            Mode =1


    # This 10ms delay helps to keep the PI from working too hard, remove it at your own risk
    time.sleep(.01)


    # To Communicate we send a value that is within the joystick deadzone to request a certain byte from the microcontroller
      # because the microcontroller is enslaved to the Raspberry pi, it cannot send it's own messages, this script will
      # read from the microcontroller when it is programmed to
    spi.writebytes([MotionIN])
    MobileAck = spi.readbytes(1)
    spi.writebytes([128])         #Ask for EncoderByte1 by the time we read
    RotationByte0= spi.readbytes(1)
    spi.writebytes([129])         #Ask for EncoderByte0 by the time we read
    RotationByte1 = spi.readbytes(1)
    spi.writebytes([130])         #Ask for PositionByte1 by the time we read
    PositionByte0 = spi.readbytes(1)
    spi.writebytes([131])         #Ask for PositionByte0 by the time we read
    PositionByte1 = spi.readbytes(1)
    spi.writebytes([132])         #Ask for PositionByte1 by the time we read
    PositionByte2 = spi.readbytes(1)
    spi.writebytes([133])         #Ask for PositionByte0 by the time we read
    PositionByte3 = spi.readbytes(1)
    
    #We may need to create a couple more commands,
    #FULL_STOP to stop crane no matter what it is doing, for shutting down crane on error,


    # This is just unpacking the bytes we recived into an 32bit number.
    RawRotation = RotationByte0[0] + (RotationByte1[0]<<8)
    RawPosition = PositionByte0[0] + (PositionByte1[0]<<8) + (PositionByte2[0]<<16) + (PositionByte3[0]<<24)
    
    # Filter out any glitches from communication
    if abs(RawRotation-RawRotation_Prev) > 120 and Rot_FirstNonZero ==1:
        RawRotation = RawRotation_Prev

    if Mode == 0: #Only Allow Calibrataion in Manual Mode
        # Recentering code, leave it on manual and let the pendulum rest, This creates an offset between what the
        #    microcontroller thinks is 0 degrees and what the python program considers 0 degrees,
        if RawRotation == RawRotation_Prev:
            NewCenterCycles = NewCenterCycles + 1
        else:
            NewCenterCycles = 0

        # this gives the processing program at least 10 cycles to realize we have waited more than 200 cycles..
        if NewCenterCycles >= 210:
            RawRotationCenter = RawRotation
            NewCenterCycles = 0
            CenterCalibDone = 1
    #This line needs to be here to record previous position for Glitch Detection
    RawRotation_Prev = RawRotation

    #Python Startup Center based on pendulum position at this script start time
    if RawRotation !=0 and Rot_FirstNonZero == 0 and CenterCalibDone == 0:
        RawRotationCenter = RawRotation
        Rot_FirstNonZero = 1

    # This returns Rotation, which is centered where we have calibrated, also outputs rotation in degrees for the processing code.
    if Rot_FirstNonZero == 1:
        Rotation = RawRotation - RawRotationCenter # 0 to 4096 ticks
        DegreesRotation = Rotation*0.087 #360/1024    To Five digits for now
        #RadiansRotation = Rotation*0.00614*0.25 #2pi/1024
    

     # Filter out any glitches from communication
    if abs(RawPosition- RawPosition_Prev) > 2000 and Pos_FirstNonZero ==1: 
        RawPosition = RawPosition_Prev
    
    if Mode == 0 and  Pos_FirstNonZero == 1: #Only Allow Calibration in Manual Mode
        # LeftLimitCreation Code
            # If we trigger left limit long enough, then we can record this position as the Left Limit
        if PrevOut < 118  and RawPosition == RawPosition_Prev and LeftCalibDone == 0:
            NewLeftLimitCycles = NewLeftLimitCycles+1
        else:
            NewLeftLimitCycles =0
        
        if NewLeftLimitCycles >= 230:
            LeftLimit = RawPosition
            NewLeftLimitCycles =0
            LeftCalibDone =1
            if RightLimit != 62722:
                RawPositionCenter = int((RightLimit - LeftLimit)/2) + LeftLimit
                PositionSelector = RawPositionCenter
                SetPoint_PositionSelector = PositionSelector
                
        # RightLimitCreation Code
            # If we trigger right limit long enough, then we can record this position as the Left Limit
        if PrevOut > 137 and RawPosition == RawPosition_Prev and RightCalibDone == 0:
            NewRightLimitCycles = NewRightLimitCycles+1
        else:
            NewRightLimitCycles =0
            
        if NewRightLimitCycles >= 230:
            RightLimit = RawPosition
            NewRightRightCycles =0
            RightCalibDone =1
            if LeftLimit != 0:
                RawPositionCenter = int((RightLimit - LeftLimit)/2) + LeftLimit
                PositionSelector = RawPositionCenter;
                SetPoint_PositionSelector = PositionSelector;
    # This line needs to be here to record previous position for Glitch Detection
    RawPosition_Prev = RawPosition

    if RawPosition !=0 and Pos_FirstNonZero == 0: #Center based on motor position at startup OF THIS PROGRAM, NOT THE ATMEGA
        RawPositionCenter = RawPosition
        Pos_FirstNonZero = 1

    if Pos_FirstNonZero == 1:
        Position = RawPosition - RawPositionCenter
        # 0 to around 67229 ticks

    # This code translates the PID output into an Joystick output with deadzone,
    PIDResult = PendulumController.Compute(0,DegreesRotation,False)
    if PIDResult > .08:
        PIDJoy =  int(137+(117*PIDResult))
    elif PIDResult < -.08:
        PIDJoy =  int(118+(117*PIDResult)) 
    elif (PIDResult < .08) and (PIDResult >-.08):
        PIDJoy = 127# This means stop
    # Remember, the Atmega uses these values to move the crane
    # 1- 118  |  127  | 138 - 255
    if PIDJoy > 255:
         PIDJoy = 255  
    if PIDJoy < 1:
         PIDJoy = 1

    # This code translates the PID output into an Joystick output with deadzone,
    PosPIDResult = PositionController.Compute(SetPoint_PositionSelector,RawPosition,True)
    if PosPIDResult > 5:
        PosPIDJoy =  int(137+(2*1.17*PosPIDResult))
    elif PosPIDResult < -5:
        PosPIDJoy =  int(118+(2*.117*PosPIDResult)) 
    elif (PosPIDResult < 5) and (PosPIDResult > -5):
        PosPIDJoy = 127# This means stop
    # Remember, the Atmega uses these values to move the crane
    # 1- 118  |  127  | 138 - 255
    if PosPIDJoy > 255:
         PosPIDJoy = 255  
    if PosPIDJoy < 1:
         PosPIDJoy = 1
   
    # This is the data sent to the PythonComms code via one string, add data to the string and read it from PythonComms
    sendstring = str(Position)+','+str(Rotation)+','+str(PrevOut)+','+str(PIDJoy)+','+str(Mode)+','+str(NewCenterCycles)+','+str(NewLeftLimitCycles)+','+str(NewRightLimitCycles)+','+str(LeftLimit)+','+str(RightLimit)+','+str(PositionSelector)+','+str(PosPIDJoy)+','+str(SetPoint_PositionSelector)+'\r\n'
    try:
        connection.send(sendstring.encode('utf-8'))    
    except:
        spi.writebytes([134])# SHUT DOWN THE CRANE IF PIPE BROKEN

    # Print all Rotation Data
    #print(MobileAck,RotationByte0[0],RotationByte1[0],RawRotation,Rotation,PrevOut)
    
    # Print refined Rotation Data
    #print(MobileAck,DegreesRotation)
    
    # Print all Position Data
    #print(MobileAck,Position)

    # These linux commands can be used to log the print statements to a file.
    # >out.csv used to overwrite
    # >>out.csv used to append

    #Approximate time per duplex (Master Send, Slave Recieve, Slave Add, Master Read)
    # .0000612 Seconds... still in microseconds!! 61 to be exact..
    # When doing only this task, the rpi is at 60% CPU..
    
    # Above is the SPI communication speed, not neccessarily the raspberry pi's output
    #good data speed, I ran into issues with an messy PWM and found out that it was the print
    #statements in this while loop that were causing the disruptions, so to give the pi time
    #to do other things I added an sleep for 1 millisecond, that smooths out the output
    
    
    
    