/*
 * SeniorProject_SPI_Communication.c
 *
 * Created: 2/1/2018 12:33:46 PM
 * Author : AndrewSchulz
 */ 

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdint.h>

uint8_t RecvByte =0;
uint8_t SendByte =0;
void init_spi_slave(void)
{
	//http://maxembedded.com/2013/11/the-spi-of-the-avr/
	// Information for SPI communication on an microcontroller.
	// For now I am assuming we want the microcontroller to be the slave
	SPCR = 0b11000000; // set spi interrupts, set spi enabled,
	DDRB = 0b00010000; // set MISO as output
}

void init_pwm(void)
{

// We want to use timer0  since it's pwm outputs are at either PD5 or PD6

// WGM02,WGM01,WGM00 = 001 for PWM,Phase Correct,   TOP at 0xFF  update OCR0x at TOP 
// WGM02,WGM01,WGM00 = 011 for PWM, FAST PWM,   TOP at 0xFF  update OCR0x at BOTTOM 

//// FOR PHASE CORRECT MODE: WE ARE NOT USING THIS
//// COM0B1, COM0B0 = 
////					10 for Clear OC0B on Compare Match when up-counting, Set OC0B when down-counting 
////					11 for Set OC0B on Compare Match when up-counting, Clear OC0B when down-counting 


// FOR FAST PWM MODE:
// COM0A1, COM0A0 =
//					10 for Clear OC0B on Compare Match , set OC0B at BOTTOM (non-inverting)
//					11 for Set OC0B on Compare Match, clear 0C0B at BOTTOM (inverting mode)


// TCCR0A: COM0A1 | COM0A0 | COM0B1 | COM0B0 |   -   |   -    | WGM01 | WGM00 |
TCCR0A = 0b00100011;

// CS02,CS01,CS00
//
//001 clk/1
//010 clk/8
//011 clk/64
//100 clk/256
//101 clk/1024
//110 external clock on T0 pin, clock on falling edge
//111 external clock on T0 pin, clock on rising edge
// Since our Microcontroller is using an 8MHz Clock... 
// and the motor controller wants pwm period < 20KHz


// TCCR0B: FOC0A | FOC0B |   -   |   -   | WGM02 | CS02 | CS01 | CS00 |
TCCR0B = 0b00000010;

// TC0 Interrupt Mask Register, only if we want a manual interrupt
//TIMSK0
// TIMER0/Counter0, Is the current value of the counter.. 
//TCNT0


// FREQUENCY OF PWM = Fclk/ divider*256
// 8MHz/256*8 = 3906Hz which is less than 20KHz

// TIMER0/Counter0 Compare Register B  (PD5) // CURRENTLY USING THIS
//OCR0B
// TIMER0/Counter0 Compare Register A  (PD6) 
//OCR0A
}

void init_Ports(void)
{
    //DDRD =  0b11101111; // set PORTD for output
	//PORTD = 0b00000000;

 	//DDRC =  0b11111111; // set PORTC for output
	//PORTC = 0b00000000;


	DDRC = 0b11000111; // Set input
	PORTC= 0b00110000; // give micro switches pullup resistors

	DDRB = 0b11111110;  // Set input
	DDRD = 0b11100011; // Set input
}



//Encoder Readers
// PORTC 0-5 are good candidates since we have not used them yet at all. 
// We need to use external interrupts in order to get this data as quickly as possible with the least interference with other parts of the Code. 
// EICRA Has two external interrupts... I think we can make that work. 

// The Pendulum Encoder has the ability to tell direction as well as ticks.. 
// OUT A is Black
// OUT B is White
// If A comes before B then it is moving Counter Clockwise
// If B comes before A then it is moving Clockwise



void init_encoder(void)
{
	EICRA = 0b00000011; 
	// EICRA
	//  |None|None|None|None|ISC11|ISC10|ISC01|ISC00|
	//  |ISC#1 | ISC#0|
	//  |  0  |  0  |  Interrupt on Low
	//  |  0  |  1  |  Interrupt on Logical Change
	//  |  1  |  0  |  Interrupt on Falling Edge
	//  |  1  |  1  |  Interrupt on Rising Edge

	EIMSK = 0b00000001;
	// EIMSK
	//  |None|None|None|None|None|None|INT1|INT0|
    // INT1 = 1 enables Interrupt 1
	// INT0 = 1 enables Interrupt 0
	// AT this point INT0 and INT1 are setup, enough for one encoder


	// Now we need to setup two more interrupts for the other encoder
	PCICR = 0b00000111;
	// EICRA
	// |None|None|None|None|None|PCIE2|PCIE1|PCIE0|
	// PCIE0 = 1 enables PCIE0 for pins 0-7 through PCMSK0
	// PCIE2 = 1 enables PCIE2 for PCINT
	
	PCMSK1 = 0b00001000;
	// Bit 3 enables PCINT11 for PCIE2  ON PC3

	PCMSK0 = 0b00000001;
	// Bit 0 enables PCINT0 for PCIE0


}


// ENTIRELY ABANDON CURRENT SENSING.. too much time taken, 

/*
void init_adc(void)
{
	ADMUX = 0b11000100;
	//ADMUX
	//  |REFS1|REFS0|ADLAR|None|MUX3|MUX2|MUX1|MUX0|
	// REFS1,REFS0 = 11 for 1.1V Vref (with external capacitor???)
	// ADLAR is for left adjust vs. right adjust, no idea if useful
	// MUX3,MUX2,MUX1,MUX0 = 0100 for ADC4 


	// 10 bit resolution between AREF and GND 
	// lets use the built-in 1.1 VREF so we get a resolution of around 0.00107421875 V per bit
	// ADC = Vin*1024/VRef

	ADCSRA = 0b10000000;
	//ADSCRA
	//  |ADEN|ADSC|ADATE|ADIF|ADIE|ADPS2|ADPS1|ADPS0|
	// ADEN = 1 Enables ADC
	// ADSC = 1 starts ADC conversion, becomes zero once conversion is completed
	// ADATE = 0 Auto Trigger Enable
	// ADIF = 0, Interrupt flag for Conversion Complete
	// ADIE = 0 ADC interrupt enable
	// ADPS2,ADPS1,ADPS0 = 000 for Clk/2 for measure time of ADC 

	// ADCL Low register of ADC result 
	// ADCH High register of ADC result
};

void init_delay_timer1(void)
{
	// I need to somehow use this timer to delay for some amount of time, then trigger an interrupt, 
	// this will be used to ignore the ADC for a certain amount of time during user inputs, 
}


void start_adc_conversion(void)
{
	ADCSRA |= 0b01000000; 
}

uint16_t checkfor_adc_result(void)
{
	if((ADCSRA | 0b10111111) == 0b10111111)
	{
		return ADCL + (ADCH<<8);  
	}
	else
	{
		return 0; 
	}
}


uint8_t monitor_current(uint8_t JoystickData)
{
	static uint8_t ReadState = 0; 

		// Based on current user input, Do we care about current usage yet?
		// Will get quality data on how long current sense works later,


	// Eventually this will be used to record the previous user input to current user input and decide how long to ignore the ADC for, 
	// right now there is a flaw where the adc conversion may be started before ignoring, and then after ignoring it reads for the first time, 
	// to fix this , turn ReadState to 0 so it has to get a new ADC reading instead of seeing the old one.

	if (JoystickData > 138) // above deadzone (RIGHT)
	{
	
	}
	else if (JoystickData < 118) // below deadzone (LEFT)
	{
	
	}


	if(ReadState == 0)
	{
		start_adc_conversion(); 
		ReadState = 1;
		
	}
	else if(ReadState == 1)
	{
		uint16_t adc_out = checkfor_adc_result();  
		if(adc_out == 0)
		{
			return 0; // return faster
		}
		else
		{ 
			// lets use the built-in 1.1 VREF so we get a resolution of around 0.00107421875 V per bit
			// ADC = Vin*1024/VRef
			if(adc_out > 70)
			{
				AllowRL = 0b00000000; // bit 1 will be LeftEnable, bit 0 will be RightEnable
				// NO MOVING ALLOWED NOW
			}
		
			ReadState = 0; // Assuming we want to read next time this is run
		}
	}

	return 0;
}
*/



// At this point I am abandoning the use of the comparator and would rather use the ADC and just poll when we want data, 
/*
// Comparator for current sensing 
void init_comparator(void)
{
	ADCSRA = 0b00100000;
		// ADSCRA
		// |ADEN|ADSC|ADATE|ADIF|ADIE|ADPS2|ADPS1|ADPS0|
		// ADEN = 0 to turn off ADC
		// ADATE = 1 to turn on AutoTriggering based on trigger selected in ADCSRB

	ADCSRB = 0b00000001;
		// ADSCRB
		// |NONE|ACME|NONE|NONE|NONE|ADTS2|ADTS1|ADTS0|
		// ACME = 0  since we want AIN1 to be the negative input to the comparator and not anything else  
		// (ADTS2,ADTS1,ADTS0) = 001 for interrupt on analog comparator

	ACSR = 0b00001000;
		// ACSR
		// |ACD|ACBG|ACO|ACI|ACIE|ACIC|ACIS1|ACIS0|
		// ACD = 0 Analog Comparator Disable = false,  so enabled!
		// ACO is the Analog Comparator Output
		// ACIE = 1  is the analog comparator interrupt enable
		// ACIS1,ACIS0 = 0,0 allows the comparator interrupt to interrupt when there is an output toggle 
		//                   we only do this because on a rising edge we need to stop the motor and let it
	    //                   move only the opposite direction, and on a falling edge we need to release the motor to move either direction

	DIDR1 = 0b00000011;
	// last two digits turn off the digital input buffer for AIN1 and AIN0

		// THIS IS USED BY THE ADC NOT THE COMPARATOR
		//DIDR0 = 0b11111111;
		// |ADC7D|ADC6D|ADC5D|ADC4D|ADC3D|ADC2D|ADC1D|ADC0D|
		// used to turn off the digital buffer for pins set to 0
		// This is required in order to read the analog signals
};	
*/


uint8_t AllowRL = 0b00000011; // bit 1 will be LeftEnable, bit 0 will be RightEnable
void Bounded_MotorDriver(void)
{
	if((PINC | 0b11101111) == 0b11101111) // PC4
	{
		AllowRL = 0b00000001;
	}
	else if((PINC | 0b11011111) == 0b11011111) // PC5
	{
		AllowRL = 0b00000010;
	}
	else
	{
		AllowRL = 0b00000011;
	}
}


// VOLTAGE OF MOTOR DRIVER IS 24V
void Joystick_to_MotorDriver(uint8_t JoystickData)
{
// Right now this uses PD6 for the PWM out, 
//                     PD0 as INA	
//					   PD1 as INB
	
 // Mapping uint8_t to Left and Right magnitudes	
 //  1--------------|-128-|--------------255
 //   (117 values) 118   138     (117 values)
 
 // The pwm is an 0-255 value, so we must map 0-117 to 0-255
 // For the moment, I will use the approximation of 117*2 = 234 which is close enough for now, 
 // assuming an 20 unit deadzone for now, 

	Bounded_MotorDriver(); // THIS RESTRICTS THE MOTORS AFTER BOUNDS ARE HIT
	if (JoystickData > 138 && ((AllowRL & 0b00000010) == 0b00000010)) // above deadzone (RIGHT)
	{
		PORTD &= 0b11111110; // INA pin = 0
		PORTD |= 0b00000010; // INB pin = 1	
		OCR0B = (JoystickData-138)*2;
	}
	else if (JoystickData < 118 && ((AllowRL & 0b00000001) == 0b00000001)) // below deadzone (LEFT)
	{
		PORTD |= 0b00000001; // INA pin = 1
		PORTD &= 0b11111101; // INB pin = 0
		OCR0B = (-(JoystickData-117))*2;
	}
	else // in middle of deadzone
	{
	PORTD &= 0b11111100; 	 // INA pin = 0 INB pin = 0
	OCR0B = 0; // STOP
	}
}



int main(void)
{
	init_Ports();
	init_spi_slave();
	init_pwm();
	init_encoder();
	sei();

    while (1) 
    {
		Joystick_to_MotorDriver(RecvByte);
    }
};

volatile uint8_t EncoderByte0 = 0;
volatile uint8_t EncoderByte1 = 0;
volatile uint16_t Rotation = 32768;

volatile uint8_t PositionByte0 = 0;
volatile uint8_t PositionByte1 = 0;
volatile uint8_t PositionByte2 = 0;
volatile uint8_t PositionByte3 = 0;
volatile uint32_t Position = 2147483648; // This was moved up here so SPI can do the work to translate into Bytes

ISR(SPI_STC_vect) // SPI MESSAGE RECIEVED INTERRUPT
{
	while((SPSR&(1<<SPIF))); // wait for message to be received
	
	if(SPDR == 128)
	{
		EncoderByte0 = Rotation & 0b0000000011111111;
		SPDR = EncoderByte0;
	}
	else if(SPDR == 129)
	{
		EncoderByte1 = (Rotation & 0b1111111100000000)>>8;
		SPDR = EncoderByte1;
	}
	else if(SPDR == 130)
	{
		PositionByte0 =  Position & 0b00000000000000000000000011111111;
		SPDR = PositionByte0;
	}
	else if(SPDR == 131)
	{
		PositionByte1 = (Position & 0b00000000000000001111111100000000)>>8;
		SPDR = PositionByte1;
	}
	else if(SPDR == 132)
	{
		PositionByte2 = (Position & 0b00000000111111110000000000000000)>>16;
		SPDR = PositionByte2;
	}
	else if(SPDR == 133)
	{
		PositionByte3 = (Position & 0b11111111000000000000000000000000)>>24;
		SPDR = PositionByte3;
	}
	else if(SPDR == 134)
	{
		AllowRL = 0b00000000; // STOP THE CRANE NOW
	}
	


	else if(SPDR != 0) // I think any cycle the pi does not send an message, SPDR == 0
	{
		RecvByte = SPDR;
	}
	
};

//  I am now certain that this logic for encoder reading must be re-written after noticing the LED for moving in a direction glowing always,
//  right now we assume that one state means we have moved by +1 and another state means we have moved by -1
//  but really it should be the switching between states that determines if we have moved or not.

// This may be simplified since I think we can have multiple vectors throw an ISR,
// making multi-state code easier to write

// DESPERATION, one interrupt version
ISR(INT0_vect) // Only acts on on Rising edge of one channel  PD2
{
	// Since PD2 is always 1
	// If PD3 is 0, we are lagging
	// If PD3 is 1  we are Leading
	 
	PORTC &= 0b00000100; // C2 ON
	if((PIND & 0b00001000) == 0b00001000)
	{
		Position++;
	}
	else
	{
		Position--; 
	}
	PORTC |= 0b11111011; // C2 OFF
}



/*
ISR(INT1_vect, ISR_ALIASOF(INT0_vect));
// These are currently attached to the motor
ISR(INT0_vect) // PD2
{
	//PORTC &= 0b00000100; // C2 ON 
	PORTC |= 0b11111011; // C2 OFF
	static uint8_t PrevPosAB = 0b00000000;
					//PD3						 //PD2			  //An parentheses issue here caused the counter to only increase cause it was reading the same pin every time
	uint8_t PosAB = (PIND & 0b00001000) + (PIND & 0b00000100); // PosA is bit 1 PosB is bit 0

	//States are
	// |ROTA|ROTB|
	// | 0  | 0  | ROTAB = 0b00000000   // next state is either 10 or 01
	// | 1  | 0  | ROTAB = 0b00000010   // next state is either 00 or 11
	// | 0  | 1  | ROTAB = 0b00000001   // next state is either 00 or 11
	// | 1  | 1  | ROTAB = 0b00000011   // next state is either 10 or 01

	// this structure should have 1 increment per pulse, and is much simpler
		if (PrevPosAB == 0b00001100)
		{
			if(PosAB == 0b00001000) // A is Leading
			{
				Position++;
			}

			if(PosAB == 0b00000100) // B is Leading
			{
				Position--;
			}
		}
	PrevPosAB = PosAB;
	PORTC &= 0b00000100; // C2 ON
	//PORTC |= 0b11111011; // C2 OFF
};
*/

ISR(PCINT0_vect, ISR_ALIASOF(PCINT1_vect));
//These are currently attached to the pendulum,
// one to channel A and one to Channel B
ISR(PCINT1_vect)
{
	static uint8_t PrevRotAB = 0b00000000;
				 //PC3					   //PB0
	uint8_t RotAB = ((PINC & 0b00001000)>>2) + (PINB & 0b00000001); // RotA is bit 1 RotB is bit 0

	//States are
	// |ROTA|ROTB|
	// | 0  | 0  | ROTAB = 0b00000000   // next state is either 10 or 01
	// | 1  | 0  | ROTAB = 0b00000010   // next state is either 00 or 11
	// | 0  | 1  | ROTAB = 0b00000001   // next state is either 00 or 11
	// | 1  | 1  | ROTAB = 0b00000011   // next state is either 10 or 01

	// This structure quadruples the resolution of the pendulum encoder
	if(PrevRotAB == 0b00000000)
	{
		if(RotAB == 0b00000010) // A is Leading
		{
			Rotation++;
		}

		if(RotAB == 0b00000001) // B is Leading
		{
			Rotation--;
		}
	}
	else if (PrevRotAB == 0b00000011)
	{
		if(RotAB == 0b00000001) // A is Leading
		{
			Rotation++;
		}

		if(RotAB == 0b00000010) // B is Leading
		{
			Rotation--;
		}
	}
	else if(PrevRotAB == 0b00000010)
	{
		if(RotAB == 0b00000011) // A is Leading
		{
			Rotation++;
		}

		if(RotAB == 0b00000000) // B is Leading
		{
			Rotation--;
		}
	}
	else if(PrevRotAB == 0b00000001)
	{
		if(RotAB == 0b00000000) // A is Leading
		{
			Rotation++;
		}

		if(RotAB == 0b00000011) // B is Leading
		{
			Rotation--;
		}
	}
	PrevRotAB = RotAB;
};

// At this point I am abandoning the use of the comparator and would rather use the ADC and just poll when we want data, 
/*
ISR(ANALOG_COMP_vect)
{
	// this indicates that there was a data toggle
	//PORTC |= 0b00100000; // PC5 pin = 1
	
	if(ACSR & 0b00100000)
	{
		// This means that we are above threshold
		PORTC |= 0b00100000; // PC5 pin = 1	
	}
};
*/